/*
 * The Clear BSD License
 * Copyright (c) 2015, Freescale Semiconductor, Inc.
 * Copyright 2016-2017 NXP
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without modification,
 * are permitted (subject to the limitations in the disclaimer below) provided
 *  that the following conditions are met:
 *
 * o Redistributions of source code must retain the above copyright notice, this list
 *   of conditions and the following disclaimer.
 *
 * o Redistributions in binary form must reproduce the above copyright notice, this
 *   list of conditions and the following disclaimer in the documentation and/or
 *   other materials provided with the distribution.
 *
 * o Neither the name of the copyright holder nor the names of its
 *   contributors may be used to endorse or promote products derived from this
 *   software without specific prior written permission.
 *
 * NO EXPRESS OR IMPLIED LICENSES TO ANY PARTY'S PATENT RIGHTS ARE GRANTED BY THIS LICENSE.
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR
 * ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
 * ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */


#include "fsl_gpio.h"
#include "peripherals.h"
#include "clock_config.h"
#include "pin_mux.h"
/*******************************************************************************
 * Definitions
 ******************************************************************************/



/*******************************************************************************
 * Prototypes
 ******************************************************************************/
/*!
 * @brief delay a while.
 */
void delay(void);
void LPUART_UserCallback(LPUART_Type *base, lpuart_handle_t *handle, status_t status, void *userData);

/*******************************************************************************
 * Variables
 ******************************************************************************/
volatile bool pitch0IsrFlag = false;
volatile bool pitch1IsrFlag = false;

lpuart_handle_t g_lpuartHandle;
lpuart_config_t config;
lpuart_transfer_t xfer;
lpuart_transfer_t sendXfer;
lpuart_transfer_t receiveXfer;

uint32_t runtime;

uint8_t g_txBuffer[30] = {0};
uint8_t g_rxBuffer[6] = {0};
volatile bool rxBufferEmpty = true;
volatile bool txBufferFull = false;
volatile bool txOnGoing = false;
volatile bool rxOnGoing = false;


uint8_t LEDdutyR = 0;
uint8_t LEDdutyG = 0;
uint8_t LEDdutyB = 0;

uint8_t i=0;
tpm_chnl_pwm_signal_param_t TPM_1_pwmSignalParamsUser[] = 
{
  {
    .chnlNumber = kTPM_Chnl_2,
    .level = kTPM_LowTrue,
    .dutyCyclePercent = 0
  },
  {
    .chnlNumber = kTPM_Chnl_3,
    .level = kTPM_LowTrue,
    .dutyCyclePercent = 0
  },
  {
    .chnlNumber = kTPM_Chnl_4,
    .level = kTPM_LowTrue,
    .dutyCyclePercent = 0
  },
  {
    .chnlNumber = kTPM_Chnl_5,
    .level = kTPM_LowTrue,
    .dutyCyclePercent = 0
  }
};


/*******************************************************************************
 * Code
 ******************************************************************************/
void delay(void)
{
    volatile uint32_t i = 0;
    for (i = 0; i < 800000; ++i)
    {
        __asm("NOP"); /* delay */
    }
}

/* LPUART user callback */
void LPUART_UserCallback(LPUART_Type *base, lpuart_handle_t *handle, status_t status, void *userData)
{
    userData = userData;

    if (kStatus_LPUART_TxIdle == status)
    {
        txBufferFull = false;
        txOnGoing = false;
    }

    if (kStatus_LPUART_RxIdle == status)
    {
        rxBufferEmpty = false;
        rxOnGoing = false;
    }
}


void PIT_1_0_IRQHANDLER(void)
{
	/* Enable interrupts from channel 0 */
	//0.5s
	if(true==PIT_GetStatusFlags(PIT_1_PERIPHERAL, kPIT_Chnl_0))
	{
		PIT_ClearStatusFlags(PIT_1_PERIPHERAL, kPIT_Chnl_0, kPIT_TimerFlag);		
		pitch0IsrFlag = 1;
	}
	
	/* Enable interrupts from channel 1 */
	//1s
	if(true==PIT_GetStatusFlags(PIT_1_PERIPHERAL, kPIT_Chnl_1))
	{
		PIT_ClearStatusFlags(PIT_1_PERIPHERAL, kPIT_Chnl_1, kPIT_TimerFlag);		
		pitch1IsrFlag = 1;
	}
	
}
/*!
 * @brief Main function
 */
int main(void)
{

	/* Board pin, clock, debug console init */
	BOARD_InitBootPins();
	BOARD_InitBootPeripherals();
	BOARD_BootClockRUN();
	
	LPUART_EnableTx(LPUART_0_PERIPHERAL, true);
	LPUART_EnableRx(LPUART_0_PERIPHERAL, true);
	
	
	PIT_StartTimer(PIT_1_PERIPHERAL, kPIT_Chnl_0); //timer 0.5s
	PIT_StartTimer(PIT_1_PERIPHERAL, kPIT_Chnl_1); //timer 1s

	LPUART_TransferCreateHandle(LPUART_0_PERIPHERAL, &g_lpuartHandle, LPUART_UserCallback, NULL);
		
    while (1)
    {
			/* If RX is idle and g_rxBuffer is empty, start to read data to g_rxBuffer. */
        if (!rxOnGoing)
        {

					if((g_rxBuffer[5]==0xaa) && (g_rxBuffer[4]==0x55)) 
					{
						//data formate " XX RR GG BB 0x55 0xAA " , XX don't care , RR hex Red duty max 100, GG green , BB blue , 0x55 0xAA for identity data 
						LEDdutyR = g_rxBuffer[1];
						LEDdutyG = g_rxBuffer[2];
						LEDdutyB = g_rxBuffer[3];
						if( LEDdutyR > 100) LEDdutyR = 100;
						if( LEDdutyG > 100) LEDdutyG = 100;
						if( LEDdutyB > 100) LEDdutyB = 100;
						
						TPM_1_pwmSignalParamsUser[1].dutyCyclePercent = LEDdutyR;
						TPM_1_pwmSignalParamsUser[2].dutyCyclePercent = LEDdutyG;
						TPM_1_pwmSignalParamsUser[3].dutyCyclePercent = LEDdutyB;
						
						TPM_SetupPwm(TPM_1_PERIPHERAL, TPM_1_pwmSignalParamsUser, sizeof(TPM_1_pwmSignalParamsUser) / sizeof(tpm_chnl_pwm_signal_param_t), kTPM_EdgeAlignedPwm, 1000U, TPM_1_CLOCK_SOURCE);

						
					}
					
					receiveXfer.data = g_rxBuffer;
					receiveXfer.dataSize = 6;
          LPUART_TransferReceiveNonBlocking(LPUART_0_PERIPHERAL, &g_lpuartHandle, &receiveXfer, NULL);
					
					rxOnGoing = true;
        }
				
				if((true==pitch1IsrFlag) &&((!txOnGoing)))
				{
					// keep sending power on time each 1 sec
					pitch1IsrFlag = 0;
					
					sprintf(g_txBuffer,"time = %u\r\n", runtime);
					sendXfer.data = g_txBuffer;
					sendXfer.dataSize = strlen(g_txBuffer);
					
					LPUART_TransferSendNonBlocking(LPUART_0_PERIPHERAL, &g_lpuartHandle, &sendXfer);
					txOnGoing = true;
					runtime++;
				}
    }
}
